from dataclasses import dataclass
from typing import Any, Optional, Union
from enum import Enum

from .nodepath_parser import Lark_StandAlone


class SegmentKind(Enum):
    DOT = 'dot'
    LIT = 'lit'
    IDX = 'idx'


@dataclass
class Segment:
    kind: SegmentKind
    key: Optional[str] = None
    index: Optional[int] = None

    def __str__(self) -> str:
        ret = ''

        if self.kind == SegmentKind.DOT:
            ret = f'.{self.key}'
        elif self.kind == SegmentKind.LIT:
            ret = f"['{self.key}']"
        elif self.kind == SegmentKind.IDX:
            ret = ''  # handled below
        else:
            raise ValueError(f'Invalid segment kind: {self.kind}')

        if self.index is not None:
            ret += f'[{self.index}]'

        return ret


class NodePath:
    def __init__(self, segments: list[Segment]):
        self.segments = segments

    @classmethod
    def parse(cls, path: str) -> 'NodePath':
        if not path:
            raise ValueError('Path cannot be empty')

        if path == '.':
            return cls([])

        parser = Lark_StandAlone()

        try:
            tree = parser.parse(path)
            segments = cls._extract_segments_from_tree(tree)
            return cls(segments)
        except Exception as e:
            raise ValueError(f'Invalid path format: {path}') from e

    @staticmethod
    def _extract_segments_from_tree(tree) -> list[Segment]:
        segments = []

        for segment_node in tree.children:
            key = None
            index = None
            kind = None

            for child in segment_node.children:
                if child.data == 'dotsegment':
                    kind = SegmentKind.DOT
                    key = child.children[0].value
                elif child.data == 'litsegment':
                    kind = SegmentKind.LIT
                    key = '' if not child.children else child.children[0].value
                elif child.data == 'index':
                    if kind is None:
                        kind = SegmentKind.IDX
                    index = int(child.children[0].value)

            if key is not None or index is not None:
                segments.append(Segment(kind, key, index))

        return segments

    def concat(self, other: Union['NodePath', str]) -> 'NodePath':
        if isinstance(other, str):
            other = self.parse(other)

        if not isinstance(other, NodePath):
            raise ValueError('Can only concatenate with NodePath or string')

        combined_segments = self.segments + other.segments
        return NodePath(combined_segments)

    def __str__(self) -> str:
        if not self.segments:
            return '.'

        return ''.join(map(str, self.segments))

    def get_value(self, data: dict) -> Optional[Any]:
        if data is None:
            raise ValueError('Data cannot be None')

        if not isinstance(data, dict) and not isinstance(data, list):
            raise ValueError('Data must be a dictionary or list')

        current = data

        for segment in self.segments:
            if segment.kind == SegmentKind.IDX:
                if not isinstance(current, list) or segment.index >= len(current):
                    return None
                current = current[segment.index]
            elif segment.kind == SegmentKind.DOT or segment.kind == SegmentKind.LIT:
                if segment.key not in current:
                    return None

                current = current[segment.key]

                if segment.index is not None:
                    if not isinstance(current, list) or segment.index >= len(current):
                        return None
                    current = current[segment.index]

        return current



import json
import os
import re
from typing import Any, Callable, List, Optional
from pathlib import Path as FSPath

from .node import Node
from .result import AssertionResult, Op, CheckStatus, NoDataError, SkippedError


class Check:
    def __init__(self, name: str, description: Optional[str] = None, node: Optional[Node] = None):
        self._name = name
        self._description = description
        self._used_vars = []
        self._results = []
        self._submitted = False

        if node is None:
            try:
                path = os.environ['LUNAR_BUNDLE_PATH']
            except KeyError:
                raise ValueError('LUNAR_BUNDLE_PATH is not set')

            try:
                node = Node.from_bundle_file(FSPath(path))
            except ValueError as e:
                raise ValueError('invalid LUNAR_BUNDLE_PATH') from e
            except FileNotFoundError:
                raise ValueError(f'LUNAR_BUNDLE_PATH does not exist: {path}')

        if not isinstance(node, Node):
            raise ValueError(f'Data must be a Node instance, got {node}')

        self._path_tracker = node._path_tracker  # Reusing the Node's path tracker to avoid weird injection
        self._node = node

    # Context Manager

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        is_no_data_error = isinstance(exc_value, NoDataError)
        is_skipped_error = isinstance(exc_value, SkippedError)

        is_suppressable_no_data = is_no_data_error and not self._node._workflows_finished()
        is_suppressable_error = is_suppressable_no_data or is_skipped_error

        if exc_value is not None:
            if is_skipped_error:
                # Clear all previous results and replace with a single skipped result
                self._results = [
                    AssertionResult(
                        op=Op.SKIP,
                        args=[],
                        result=CheckStatus.SKIPPED,
                        failure_message=str(exc_value),
                    )
                ]
            elif is_suppressable_no_data:
                self._results.append(
                    AssertionResult(
                        op=Op.FAIL,
                        args=[],
                        result=CheckStatus.PENDING,
                        failure_message=str(exc_value),
                    )
                )
            elif is_no_data_error:
                self._results.append(
                    AssertionResult(
                        op=Op.FAIL,
                        args=[],
                        result=CheckStatus.ERROR,
                        failure_message=f'Unexpected error: {exc_value}',
                    )
                )
            else:
                self._results.append(
                    AssertionResult(
                        op=Op.FAIL,
                        args=[],
                        result=CheckStatus.ERROR,
                        failure_message=str(exc_value),
                    )
                )

        self._submit()
        # Suppress SkippedError and suppressable NoDataError, let others propagate
        return is_suppressable_error

    # Properties

    @property
    def name(self) -> str:
        return self._name

    @property
    def status(self) -> CheckStatus:
        if not self._results:
            return CheckStatus.PASS

        has_no_data = False
        has_fail = False
        has_error = False

        for result in self._results:
            if result.result == CheckStatus.ERROR:
                has_error = True
            if result.result == CheckStatus.PENDING:
                has_no_data = True
            if result.result == CheckStatus.FAIL:
                has_fail = True

        if has_error:
            return CheckStatus.ERROR
        if has_no_data:
            return CheckStatus.PENDING
        elif has_fail:
            return CheckStatus.FAIL

        return CheckStatus.PASS

    @property
    def failure_reasons(self) -> list:
        failure_messages = [
            result.failure_message
            for result in self._results
            if (result.result == CheckStatus.FAIL and result.failure_message is not None)
        ]
        return failure_messages

    # Root node wrappers

    def get_value(self, path: str = '.') -> Any:
        return self._node.get_value(path)

    def get_node(self, path: str) -> Node:
        return self._node.get_node(path)

    def get_all_values(self, path: str = '.') -> List[Any]:
        return self._node.get_all_values(path)

    def get_value_or_default(self, path: str = '.', default: Any = None) -> Any:
        return self._node.get_value_or_default(path, default)

    def exists(self, path: str = '.') -> bool:
        return self._node.exists(path)

    # Iteration

    def __iter__(self):
        return iter(self._node)

    def items(self):
        return self._node.items()

    # Flow control

    def fail(self, failure_message: Optional[str] = None) -> None:
        self._make_assertion(
            Op.FAIL,
            lambda v: False,
            False,
            failure_message or 'Policy Forced Failure',
        )

    def skip(self, reason: Optional[str] = None) -> None:
        raise SkippedError(reason or 'Check skipped')

    # Assertions

    def assert_true(
        self,
        value: Any,
        failure_message: Optional[str] = None,
    ) -> None:
        self._make_assertion(
            Op.TRUE,
            lambda v: v is True,
            value,
            failure_message or f'{value} is not true',
        )

    def assert_false(
        self,
        value: Any,
        failure_message: Optional[str] = None,
    ) -> None:
        self._make_assertion(
            Op.FALSE,
            lambda v: v is False,
            value,
            failure_message or f'{value} is not false',
        )

    def assert_equals(
        self,
        value: Any,
        expected: Any,
        failure_message: Optional[str] = None,
    ) -> None:
        self._make_assertion(
            Op.EQUALS,
            lambda v: v == expected,
            value,
            failure_message or f'{value} is not equal to {expected}',
        )

    def assert_contains(
        self,
        value: Any,
        expected: Any,
        failure_message: Optional[str] = None,
    ) -> None:
        self._make_assertion(
            Op.CONTAINS,
            lambda v: expected in v,
            value,
            failure_message or f'{value} does not contain {expected}',
        )

    def assert_greater(
        self,
        value: Any,
        expected: Any,
        failure_message: Optional[str] = None,
    ):
        self._make_assertion(
            Op.GREATER,
            lambda v: v > expected,
            value,
            failure_message or f'{value} is not greater than {expected}',
        )

    def assert_greater_or_equal(
        self,
        value: Any,
        expected: Any,
        failure_message: Optional[str] = None,
    ) -> None:
        self._make_assertion(
            Op.GREATER_OR_EQUAL,
            lambda v: v >= expected,
            value,
            failure_message or f'{value} is not greater than or equal to {expected}',
        )

    def assert_less(
        self,
        value: Any,
        expected: Any,
        failure_message: Optional[str] = None,
    ) -> None:
        self._make_assertion(
            Op.LESS,
            lambda v: v < expected,
            value,
            failure_message or f'{value} is not less than {expected}',
        )

    def assert_less_or_equal(
        self,
        value: Any,
        expected: Any,
        failure_message: Optional[str] = None,
    ) -> None:
        self._make_assertion(
            Op.LESS_OR_EQUAL,
            lambda v: v <= expected,
            value,
            failure_message or f'{value} is not less than or equal to {expected}',
        )

    def assert_match(
        self,
        value: Any,
        pattern: str,
        failure_message: Optional[str] = None,
    ) -> None:
        self._make_assertion(
            Op.MATCH,
            lambda v: re.match(pattern, v) is not None,
            value,
            failure_message or f'{value} does not match {pattern}',
        )

    def assert_exists(self, path: str, failure_message: Optional[str] = None) -> None:
        # This assertion behaves differently than the others, so it gets its
        # own implementation. Normal no-data behavior before workflows completed, fail afterward.

        try:
            self._make_assertion(
                Op.EXISTS,
                lambda v: v is not None,
                self.get_value(path),
                failure_message or f'{path} does not exist',
            )
        except ValueError:
            self._results.append(
                AssertionResult(
                    op=Op.FAIL,
                    args=[],
                    result=CheckStatus.FAIL,
                    failure_message=(failure_message or f'{path} does not exist'),
                )
            )

    # Private methods

    def _submit(self):
        if not self._submitted:
            output = {
                'name': self._name,
                'assertions': [result.toJson() for result in self._results],
            }

            output['paths'] = list(
                dict.fromkeys(p if p.startswith('.') else f'.{p}' for p in self._path_tracker.get_paths())
            )

            if self._description is not None:
                output['description'] = self._description

            prefix = os.environ.get('LUNAR_LOG_PREFIX', '')
            print(f'{prefix}{json.dumps(output)}')

            self._submitted = True

    def _make_assertion(
        self,
        op: Op,
        check_fn: Callable[[Any], bool],
        value: Any,
        failure_message: str,
    ):
        if value is None:
            raise NoDataError('Value was None')

        ok = check_fn(value)

        try:
            # We don't care about the actual value,
            # we just want to make sure its serializable at submission time
            serialized_value = str(value)
        except Exception as e:
            type_name = type(value).__name__
            serialized_value = f'<typename {type_name}: {e}>'

        self._results.append(
            AssertionResult(
                op=op,
                args=[serialized_value],
                result=CheckStatus.PASS if ok else CheckStatus.FAIL,
                failure_message=failure_message if not ok else None,
            )
        )



import json
import pytest

from src.lunar_policy import Check, Node


class TestCheckPaths:
    def test_path_in_check(self, capsys):
        node = Node.from_component_json(
            {
                'value': True,
            }
        )

        with Check('test', node=node) as c:
            c.assert_true(c.get_value('.value'))

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result['paths'] == ['.value']

    def test_multiple_paths_in_check(self, capsys):
        node = Node.from_component_json({'true_value': True, 'false_value': False})

        with Check('test', node=node) as c:
            v = c.get_value('.false_value')
            c.assert_false(v)
            c.assert_true(c.get_value('.true_value'))

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result['paths'] == ['.false_value', '.true_value']

    def test_paths_not_in_check(self, capsys):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.assert_true(True)

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result['paths'] == []

    def test_get_invalid_path(self):
        node = Node.from_component_json({})

        with pytest.raises(ValueError):
            with Check('test', node=node) as c:
                c.get_value('@#&*(!%)')



from src.lunar_policy.nodepath import NodePath, Segment, SegmentKind
import pytest


class TestNodePathValidSyntax:
    def test_simple_path(self):
        path = NodePath.parse('.a.b.c')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a'),
            Segment(kind=SegmentKind.DOT, key='b'),
            Segment(kind=SegmentKind.DOT, key='c'),
        ]

    def test_path_with_numbers_in_key(self):
        path = NodePath.parse('.a1.b2.c3')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a1'),
            Segment(kind=SegmentKind.DOT, key='b2'),
            Segment(kind=SegmentKind.DOT, key='c3'),
        ]

    def test_path_with_longer_names(self):
        path = NodePath.parse('.these.are.not.the.droids.you.are.looking.for')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='these'),
            Segment(kind=SegmentKind.DOT, key='are'),
            Segment(kind=SegmentKind.DOT, key='not'),
            Segment(kind=SegmentKind.DOT, key='the'),
            Segment(kind=SegmentKind.DOT, key='droids'),
            Segment(kind=SegmentKind.DOT, key='you'),
            Segment(kind=SegmentKind.DOT, key='are'),
            Segment(kind=SegmentKind.DOT, key='looking'),
            Segment(kind=SegmentKind.DOT, key='for'),
        ]

    def test_path_with_trailing_index(self):
        path = NodePath.parse('.a.b[0]')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a'),
            Segment(kind=SegmentKind.DOT, key='b', index=0),
        ]

    def test_path_with_inner_index(self):
        path = NodePath.parse('.a.b[0].c')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a'),
            Segment(kind=SegmentKind.DOT, key='b', index=0),
            Segment(kind=SegmentKind.DOT, key='c'),
        ]

    def test_path_with_multiple_indexes(self):
        path = NodePath.parse('.a.b[0].c[1]')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a'),
            Segment(kind=SegmentKind.DOT, key='b', index=0),
            Segment(kind=SegmentKind.DOT, key='c', index=1),
        ]

    def test_path_with_multiple_consecutive_indexes(self):
        path = NodePath.parse('.a.b[0].c[1][2]')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a'),
            Segment(kind=SegmentKind.DOT, key='b', index=0),
            Segment(kind=SegmentKind.DOT, key='c', index=1),
            Segment(kind=SegmentKind.IDX, index=2),
        ]

    def test_root_path(self):
        path = NodePath.parse('.')
        assert path.segments == []

    def test_literal_segment(self):
        path = NodePath.parse(".a['!#$ %&'].c")
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a'),
            Segment(kind=SegmentKind.LIT, key='!#$ %&'),
            Segment(kind=SegmentKind.DOT, key='c'),
        ]

    def test_literal_segment_with_index(self):
        path = NodePath.parse(".a['~'][0].c")
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a'),
            Segment(kind=SegmentKind.LIT, key='~', index=0),
            Segment(kind=SegmentKind.DOT, key='c'),
        ]

    def test_literal_multiple_segments(self):
        path = NodePath.parse("['a']['b']['c']")
        assert path.segments == [
            Segment(kind=SegmentKind.LIT, key='a'),
            Segment(kind=SegmentKind.LIT, key='b'),
            Segment(kind=SegmentKind.LIT, key='c'),
        ]

    def test_underscore_allowed_in_key(self):
        path = NodePath.parse('.a._b.c')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='a'),
            Segment(kind=SegmentKind.DOT, key='_b'),
            Segment(kind=SegmentKind.DOT, key='c'),
        ]

    def test_underscore_allowed_start_of_key(self):
        path = NodePath.parse('._a.b')
        assert path.segments == [
            Segment(kind=SegmentKind.DOT, key='_a'),
            Segment(kind=SegmentKind.DOT, key='b'),
        ]

    def test_path_with_escaped_quotes_in_literal(self):
        path = NodePath.parse("['a\\'b']")
        assert path.segments == [Segment(kind=SegmentKind.LIT, key="a\\'b")]

    def test_path_with_backslash_in_literal(self):
        path = NodePath.parse("['a\\\\b']")
        assert path.segments == [Segment(kind=SegmentKind.LIT, key='a\\\\b')]

    def test_path_with_empty_literal(self):
        path = NodePath.parse("['']")
        assert path.segments == [Segment(kind=SegmentKind.LIT, key='')]

    def test_path_with_very_long_index(self):
        path = NodePath.parse('.a[999999999]')
        assert path.segments == [Segment(kind=SegmentKind.DOT, key='a', index=999999999)]

    def test_path_with_unicode_characters(self):
        path = NodePath.parse("['café']['ñoño']")
        assert path.segments == [
            Segment(kind=SegmentKind.LIT, key='café'),
            Segment(kind=SegmentKind.LIT, key='ñoño'),
        ]

    def test_path_with_array_root(self):
        path = NodePath.parse('[0]')
        assert path.segments == [Segment(kind=SegmentKind.IDX, index=0)]

    def test_nested_rooted_arrays(self):
        path = NodePath.parse('[0][1]')
        assert path.segments == [
            Segment(kind=SegmentKind.IDX, index=0),
            Segment(kind=SegmentKind.IDX, index=1),
        ]


class TestNodePathInvalidSyntax:
    def test_empty_path(self):
        with pytest.raises(ValueError):
            NodePath.parse('')

    def test_path_without_leading_period(self):
        with pytest.raises(ValueError):
            NodePath.parse('a.b.c')

    def test_path_with_empty_segment(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a..b')

    def test_path_with_open_bracket(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a.b[')

    def test_path_with_close_bracket(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a.b]')

    def test_path_with_invalid_index(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a.b[notanindex]')

    def test_path_with_invalid_characters(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a.b-')

    def test_path_with_extra_opening_bracket(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a.b[[0]')

    def test_path_with_extra_closing_bracket(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a.b[0]]')

    def test_path_with_whitespace(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a.be cool')

    def test_path_with_numbers_only_in_key(self):
        with pytest.raises(ValueError):
            NodePath.parse('.1.2.3')

    def test_path_start_with_numbers(self):
        with pytest.raises(ValueError):
            NodePath.parse('.1a.2b.3c')

    def test_path_with_unicode_characters(self):
        with pytest.raises(ValueError):
            NodePath.parse('.café.ñoño')

    def test_path_index_wildcard(self):
        with pytest.raises(ValueError):
            NodePath.parse('.a[*]')


class TestNodePathGetValueInvalidValue:
    def test_none_value(self):
        with pytest.raises(ValueError):
            NodePath.parse(None)


class TestNodePathConcat:
    def test_concat_simple_paths(self):
        path1 = NodePath.parse('.a.b')
        path2 = NodePath.parse('.c.d')
        combined = path1.concat(path2)
        assert str(combined) == '.a.b.c.d'

    def test_concat_path_with_trailing_index(self):
        path1 = NodePath.parse('.a.b[0]')
        path2 = NodePath.parse('.c.d')
        combined = path1.concat(path2)
        assert str(combined) == '.a.b[0].c.d'

    def test_concat_path_with_inner_index(self):
        path1 = NodePath.parse('.a.b[0].c')
        path2 = NodePath.parse('.d.e')
        combined = path1.concat(path2)
        assert str(combined) == '.a.b[0].c.d.e'

    def test_concat_path_both_with_indexes(self):
        path1 = NodePath.parse('.a.b[0].c[1]')
        path2 = NodePath.parse('.d.e[2]')
        combined = path1.concat(path2)
        assert str(combined) == '.a.b[0].c[1].d.e[2]'

    def test_concat_path_with_string(self):
        path1 = NodePath.parse('.a.b')
        path2 = '.c.d'
        combined = path1.concat(path2)
        assert str(combined) == '.a.b.c.d'

    def test_concat_path_with_literal_string(self):
        path1 = NodePath.parse('.a.b')
        path2 = "['c'].d"
        combined = path1.concat(path2)
        assert str(combined) == ".a.b['c'].d"

    def test_concat_path_with_right_literal(self):
        path1 = NodePath.parse('.a.b')
        path2 = NodePath.parse("['c'].d")
        combined = path1.concat(path2)
        assert str(combined) == ".a.b['c'].d"

    def test_concat_path_with_left_literal(self):
        path1 = NodePath.parse("['a'].b")
        path2 = NodePath.parse('.c.d')
        combined = path1.concat(path2)
        assert str(combined) == "['a'].b.c.d"

    def test_concat_path_with_both_literals(self):
        path1 = NodePath.parse("['a'].b")
        path2 = NodePath.parse("['c'].d")
        combined = path1.concat(path2)
        assert str(combined) == "['a'].b['c'].d"

    def test_concat_path_only_literals(self):
        path1 = NodePath.parse("['a']")
        path2 = NodePath.parse("['b']")
        combined = path1.concat(path2)
        assert str(combined) == "['a']['b']"

    def test_concat_root_path_left(self):
        path1 = NodePath.parse('.')
        path2 = NodePath.parse('.a.b')
        combined = path1.concat(path2)
        assert str(combined) == '.a.b'

    def test_concat_root_path_right(self):
        path1 = NodePath.parse('.a.b')
        path2 = NodePath.parse('.')
        combined = path1.concat(path2)
        assert str(combined) == '.a.b'

    def test_concat_root_path_both(self):
        path1 = NodePath.parse('.')
        path2 = NodePath.parse('.')
        combined = path1.concat(path2)
        assert str(combined) == '.'

    def test_concat_path_with_array_root(self):
        path1 = NodePath.parse('[0]')
        path2 = NodePath.parse('.a.b')
        combined = path1.concat(path2)
        assert str(combined) == '[0].a.b'

    def test_concat_path_with_array_root_and_literal(self):
        path1 = NodePath.parse('[0]')
        path2 = NodePath.parse("['a'].b")
        combined = path1.concat(path2)
        assert str(combined) == "[0]['a'].b"

    def test_concat_two_array_roots(self):
        path1 = NodePath.parse('[0]')
        path2 = NodePath.parse('[1]')
        combined = path1.concat(path2)
        assert str(combined) == '[0][1]'

    def test_concat_literal_with_index_and_array_root(self):
        path1 = NodePath.parse("['a'][0]")
        path2 = NodePath.parse('[1]')
        combined = path1.concat(path2)
        assert str(combined) == "['a'][0][1]"


class TestNodePathConcatInvalidValue:
    def test_concat_none_value(self):
        path1 = NodePath.parse('.')
        with pytest.raises(ValueError):
            path1.concat(None)

    def test_concat_invalid_path(self):
        path1 = NodePath.parse('.')
        with pytest.raises(ValueError):
            path1.concat('invalid path')

    def test_concat_invalid_type(self):
        path1 = NodePath.parse('.')
        with pytest.raises(ValueError):
            path1.concat(123)


getValueTestObject = {
    'root_value': 'root_value',
    'nested': {'value': 'nested_value'},
    'array_string': ['a', 'b', 'c'],
    'array_object': [{'obj1': 'obj1_value'}, {'obj2': 'obj2_value'}],
    '!@#_ $*[]': 'special_value',
    '****': {'****': 'nested_special_value'},
    '': 'empty_string_value',
    'nested_array': [
        'hello',
        ['nested_array_value', {'d': 'nested_array_object_value'}],
    ],
}

getValueTestArray = [1, 'hello', {'a': 'b', 'c': [1, 'hello']}]


class TestNodePathGetValue:
    def test_get_value_simple_path(self):
        path = NodePath.parse('.root_value')
        assert path.get_value(getValueTestObject) == 'root_value'

    def test_get_value_nested_path(self):
        path = NodePath.parse('.nested.value')
        assert path.get_value(getValueTestObject) == 'nested_value'

    def test_get_value_missing_path(self):
        path = NodePath.parse('.missing')
        assert path.get_value(getValueTestObject) is None

    def test_get_value_array_string_path(self):
        path = NodePath.parse('.array_string[0]')
        assert path.get_value(getValueTestObject) == 'a'

    def test_get_value_array_string_path_out_of_bounds(self):
        path = NodePath.parse('.array_string[3]')
        assert path.get_value(getValueTestObject) is None

    def test_get_value_array_object_path(self):
        path = NodePath.parse('.array_object[1].obj2')
        assert path.get_value(getValueTestObject) == 'obj2_value'

    def test_get_value_special_characters_path(self):
        path = NodePath.parse("['!@#_ $*[]']")
        assert path.get_value(getValueTestObject) == 'special_value'

    def test_get_value_nested_special_characters_path(self):
        path = NodePath.parse("['****']['****']")
        assert path.get_value(getValueTestObject) == 'nested_special_value'

    def test_get_value_empty_string_path(self):
        path = NodePath.parse("['']")
        assert path.get_value(getValueTestObject) == 'empty_string_value'

    def test_get_value_nested_array_path(self):
        path = NodePath.parse('.nested_array[1][0]')
        assert path.get_value(getValueTestObject) == 'nested_array_value'

    def test_get_value_nested_array_object_path(self):
        path = NodePath.parse('.nested_array[1][1].d')
        assert path.get_value(getValueTestObject) == 'nested_array_object_value'

    def test_get_value_array_path(self):
        path = NodePath.parse('[0]')
        assert path.get_value(getValueTestArray) == 1

    def test_get_value_array_path_out_of_bounds(self):
        path = NodePath.parse('[3]')
        assert path.get_value(getValueTestArray) is None

    def test_get_value_array_path_nested_object_path(self):
        path = NodePath.parse('[2].c[0]')
        assert path.get_value(getValueTestArray) == 1


class TestNodePathGetValueInvalidData:
    def test_get_value_none_value(self):
        with pytest.raises(ValueError):
            NodePath.parse('.').get_value(None)

    def test_get_value_non_dict_value(self):
        with pytest.raises(ValueError):
            NodePath.parse('.').get_value('not a dict')



import pytest
import json
import tempfile
from pathlib import Path as FSPath
from src.lunar_policy import Node, NodePath, NoDataError
from src.lunar_policy.node import NodePathTracker


class TestDataLoading:
    def test_from_component_json(self):
        merged_json = {'string': 'hello world'}
        data = Node.from_component_json(merged_json)
        assert data.get_value('.string') == 'hello world'
        assert data.get_all_values('.string') == ['hello world']

    def test_from_bundle_json(self):
        test_data = {
            'bundle_info': {},
            'metadata_instances': [{'payload': {'string': 'hello world'}}],
            'merged_blob': {'string': 'hello world'},
        }
        data = Node.from_bundle_json(test_data)
        assert data.get_value('.string') == 'hello world'
        assert data.get_all_values('.string') == ['hello world']

    def test_from_bundle_json_missing_bundle_info(self):
        test_data = {'metadata_instances': [], 'merged_blob': {}}
        with pytest.raises(ValueError):
            Node.from_bundle_json(test_data)

    def test_from_bundle_json_missing_metadata_instances(self):
        test_data = {'bundle_info': {}, 'merged_blob': {}}
        with pytest.raises(ValueError):
            Node.from_bundle_json(test_data)

    def test_from_bundle_json_missing_merged_blob(self):
        test_data = {'bundle_info': {}, 'metadata_instances': []}
        with pytest.raises(ValueError):
            Node.from_bundle_json(test_data)

    def test_from_bundle_file(self):
        test_data = {
            'bundle_info': {},
            'metadata_instances': [{'payload': {'string': 'hello world'}}],
            'merged_blob': {'string': 'hello world'},
        }

        with tempfile.NamedTemporaryFile('w+', delete=False, suffix='.json') as tmpfile:
            tmpfile.write(json.dumps(test_data))
            tmpfile.flush()
            tmp_path = FSPath(tmpfile.name)

        data = Node.from_bundle_file(FSPath(tmp_path))
        assert data.get_value('.string') == 'hello world'
        assert data.get_all_values('.string') == ['hello world']

    def test_from_file_invalid_path(self):
        with pytest.raises(FileNotFoundError):
            Node.from_bundle_file(FSPath('/my/path/does_not_exist.json'))

    def test_from_file_relative_path(self):
        with pytest.raises(ValueError):
            Node.from_bundle_file(FSPath('relative/path.json'))

    def test_from_file_invalid_json(self):
        test_data = '}'

        with tempfile.NamedTemporaryFile('w+', delete=False, suffix='.json') as tmpfile:
            tmpfile.write(test_data)
            tmpfile.flush()
            tmp_path = FSPath(tmpfile.name)

        with pytest.raises(json.JSONDecodeError):
            Node.from_bundle_file(FSPath(tmp_path))


class TestGetMerged:
    @pytest.fixture
    def snippet_data(self):
        test_data = {
            'bundle_info': {},
            'metadata_instances': [],
            'merged_blob': {
                'string': 'hello world',
                'array': ['hello', 'world'],
                'object1': {'hello': 'world'},
                'object2': {'hello': 'moon'},
                'empty_object': {},
                'empty_array': [],
            },
        }
        return Node.from_bundle_json(test_data)

    def test_get_single_value(self, snippet_data):
        assert snippet_data.get_value('.string') == 'hello world'

    def test_get_missing(self, snippet_data):
        with pytest.raises(NoDataError):
            snippet_data.get_value('.missing')

    def test_get_missing_workflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        with pytest.raises(ValueError):
            snippet_data.get_value('.missing')

    def test_get_array(self, snippet_data):
        assert snippet_data.get_value('.array') == ['hello', 'world']

    def test_get_array_index(self, snippet_data):
        assert snippet_data.get_value('.array[0]') == 'hello'

    def test_get_nested_object(self, snippet_data):
        assert snippet_data.get_value('.object1.hello') == 'world'

    def test_get_nested_object_missing(self, snippet_data):
        with pytest.raises(NoDataError):
            snippet_data.get_value('.object1.missing')

    def test_get_nested_object_missing_workflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        with pytest.raises(ValueError):
            snippet_data.get_value('.object1.missing')

    def test_get_empty_object(self, snippet_data):
        assert snippet_data.get_value('.empty_object') == {}

    def test_get_empty_array(self, snippet_data):
        assert snippet_data.get_value('.empty_array') == []


class TestGetAll:
    @pytest.fixture
    def snippet_data(self):
        test_data = {
            'bundle_info': {},
            'merged_blob': {},
            'metadata_instances': [
                {
                    'payload': {
                        'single': 'hello world',
                        'double': 'hello1',
                        'single_array': ['hello', 'world'],
                        'double_array': ['goodbye', 'moon'],
                        'single_object': {'hello': 'world'},
                        'double_object': {'hello': 'moon'},
                    }
                },
                {
                    'payload': {
                        'double': 'hello2',
                        'double_array': ['mars', 'i', 'guess'],
                        'double_object': {'hello': 'venus'},
                    }
                },
                {'not_in_any_delta': 'hello pluto'},
            ],
        }
        return Node.from_bundle_json(test_data)

    def test_get_single_key_in_one_delta(self, snippet_data):
        assert snippet_data.get_all_values('.single') == ['hello world']

    def test_get_single_key_in_two_deltas(self, snippet_data):
        assert snippet_data.get_all_values('.double') == ['hello2', 'hello1']

    def test_get_missing(self, snippet_data):
        with pytest.raises(NoDataError):
            snippet_data.get_all_values('.missing')

    def test_get_missing_workflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        with pytest.raises(ValueError):
            snippet_data.get_all_values('.missing')

    def test_get_single_array(self, snippet_data):
        assert snippet_data.get_all_values('.single_array') == [['hello', 'world']]

    def test_get_single_array_index(self, snippet_data):
        assert snippet_data.get_all_values('.single_array[0]') == ['hello']

    def test_get_double_array(self, snippet_data):
        assert snippet_data.get_all_values('.double_array') == [
            ['mars', 'i', 'guess'],
            ['goodbye', 'moon'],
        ]

    def test_get_double_array_index(self, snippet_data):
        assert snippet_data.get_all_values('.double_array[0]') == ['mars', 'goodbye']

    def test_get_double_array_index_some_out_of_bounds(self, snippet_data):
        assert snippet_data.get_all_values('.double_array[2]') == ['guess']

    def test_get_single_object(self, snippet_data):
        assert snippet_data.get_all_values('.single_object') == [{'hello': 'world'}]

    def test_get_double_object(self, snippet_data):
        assert snippet_data.get_all_values('.double_object') == [
            {'hello': 'venus'},
            {'hello': 'moon'},
        ]

    def test_get_single_object_key(self, snippet_data):
        assert snippet_data.get_all_values('.single_object.hello') == ['world']

    def test_get_double_object_key(self, snippet_data):
        assert snippet_data.get_all_values('.double_object.hello') == ['venus', 'moon']

    def test_get_single_object_key_missing(self, snippet_data):
        with pytest.raises(NoDataError):
            snippet_data.get_all_values('.single_object.missing')

    def test_get_single_object_key_missing_wflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        with pytest.raises(ValueError):
            snippet_data.get_all_values('.single_object.missing')

    def test_get_double_object_key_missing(self, snippet_data):
        with pytest.raises(NoDataError):
            snippet_data.get_all_values('.double_object.missing')

    def test_get_double_object_key_missing_wflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        with pytest.raises(ValueError):
            snippet_data.get_all_values('.double_object.missing')

    def test_get_not_in_any_delta(self, snippet_data):
        with pytest.raises(NoDataError):
            snippet_data.get_all_values('.not_in_any_delta')

    def test_get_not_in_any_delta_workflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        with pytest.raises(ValueError):
            snippet_data.get_all_values('.not_in_any_delta')


class TestNodeInvalidData:
    def test_none_data(self):
        with pytest.raises(ValueError):
            Node(NodePath.parse('.'), None, NodePathTracker())

    def test_empty_data(self):
        with pytest.raises(ValueError):
            Node(NodePath.parse('.'), {}, NodePathTracker())

    def test_invalid_snippet_data_missing_merged_blob(self):
        with pytest.raises(ValueError):
            Node(NodePath.parse('.'), {'metadata_instances': []}, NodePathTracker())

    def test_invalid_snippet_data_missing_metadata_instances(self):
        with pytest.raises(ValueError):
            Node(NodePath.parse('.'), {'merged_blob': {}}, NodePathTracker())

    def test_no_path_tracker(self):
        with pytest.raises(ValueError):
            Node(NodePath.parse('.'), {'merged_blob': {}, 'metadata_instances': []}, None)


class TestExists:
    @pytest.fixture
    def snippet_data(self):
        test_data = {
            'bundle_info': {},
            'merged_blob': {
                'a': 'hello',
                'b': ['hello', 'world'],
                'c': {'hello': 'world'},
            },
            'metadata_instances': [],
        }
        return Node.from_bundle_json(test_data)

    def test_exists_on_scalar(self, snippet_data):
        assert snippet_data.exists()

    def test_exists_on_object(self, snippet_data):
        assert snippet_data.exists('.a')

    def test_exists_on_array_index(self, snippet_data):
        assert snippet_data.exists('.b[0]')

    def test_exists_on_object_key(self, snippet_data):
        assert snippet_data.exists('.c.hello')

    def test_exists_on_missing(self, snippet_data):
        with pytest.raises(NoDataError):
            snippet_data.exists('.d.hello')

    def test_exists_on_missing_workflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        assert not snippet_data.exists('.d.hello')

    def test_exists_on_sub_node(self, snippet_data):
        assert snippet_data.get_node('.c').exists()


class TestGetNode:
    @pytest.fixture
    def snippet_data(self):
        test_data = {
            'bundle_info': {},
            'merged_blob': {
                'a': {'b': {'c': 'hello world'}},
                'd': ['hello', ['world', 'mars']],
            },
            'metadata_instances': [
                {'payload': {'a': {'b': {'c': 'data1'}}}},
                {'payload': {'a': {'b': {'c': 'data2'}}}},
            ],
        }
        return Node.from_bundle_json(test_data)

    def test_get_value_from_sub_node(self, snippet_data):
        sub_node = snippet_data.get_node('.a.b')
        assert sub_node.get_value('.c') == 'hello world'

    def test_get_all_values_from_sub_node(self, snippet_data):
        sub_node = snippet_data.get_node('.a.b')
        assert sub_node.get_all_values('.c') == ['data2', 'data1']

    def test_get_value_from_array_sub_node(self, snippet_data):
        sub_node = snippet_data.get_node('.d[0]')
        assert sub_node.get_value() == 'hello'

    def test_get_value_from_array_sub_node_index(self, snippet_data):
        sub_node = snippet_data.get_node('.d[1]')
        assert sub_node.get_value('[0]') == 'world'

    def test_get_node_on_missing(self, snippet_data):
        node = snippet_data.get_node('.missing')
        with pytest.raises(NoDataError):
            node.get_value()

    def test_get_node_on_invalid_path(self, snippet_data):
        with pytest.raises(ValueError):
            snippet_data.get_node('.1]')


class TestNodeIteration:
    @pytest.fixture
    def snippet_data(self):
        test_data = {
            'bundle_info': {},
            'merged_blob': {
                'a': 'hello',
                'b': 'world',
                'c': ['hello', ['world', 'mars']],
                'd': {},
                'e': [],
            },
            'metadata_instances': [],
        }
        return Node.from_bundle_json(test_data)

    def test_iterate_over_object(self, snippet_data):
        assert list(snippet_data) == ['a', 'b', 'c', 'd', 'e']

    def test_iterate_over_array(self, snippet_data):
        items = list(snippet_data.get_node('.c'))
        for item in items:
            assert isinstance(item, Node)

        assert [item.get_value() for item in items] == ['hello', ['world', 'mars']]

    def test_iterate_over_scalar(self, snippet_data):
        with pytest.raises(ValueError):
            list(snippet_data.get_node('.a'))

    def test_iterate_dict_items(self, snippet_data):
        items = list(snippet_data.items())
        for key, node in items:
            assert isinstance(key, str)
            assert isinstance(node, Node)

        assert [node.get_value() for key, node in items] == [
            'hello',
            'world',
            ['hello', ['world', 'mars']],
            {},
            [],
        ]

    def test_iterate_dict_items_on_non_dict(self, snippet_data):
        with pytest.raises(ValueError):
            list(snippet_data.get_node('.a').items())

    def test_iterate_dict_items_on_empty_dict(self, snippet_data):
        items = list(snippet_data.get_node('.d').items())
        assert len(items) == 0

    def test_iterate_on_empty_array(self, snippet_data):
        items = list(snippet_data.get_node('.e'))
        assert len(items) == 0


class TestGetValueOrDefault:
    @pytest.fixture
    def snippet_data(self):
        test_data = {
            'a': 'hello',
        }
        return Node.from_component_json(test_data)

    def test_get_value_or_default_on_existing(self, snippet_data):
        assert snippet_data.get_value_or_default('.a', 'default') == 'hello'

    def test_get_value_or_default_on_existing_workflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        assert snippet_data.get_value_or_default('.a', 'default') == 'hello'

    def test_get_value_or_default_on_missing(self, snippet_data):
        assert snippet_data.get_value_or_default('.b', 'default') == 'default'

    def test_get_value_or_default_on_missing_workflows_finished(self, snippet_data):
        snippet_data._data['bundle_info']['workflows_finished'] = True
        assert snippet_data.get_value_or_default('.b', 'default') == 'default'

    def test_get_value_or_default_none_default(self, snippet_data):
        assert snippet_data.get_value_or_default('.b') is None



from src.lunar_policy import Check, CheckStatus, Node


class TestCheckName:
    def test_name(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.assert_true(True)

        assert c.name == 'test'


class TestCheckFailureReasons:
    def test_failure_reasons_single(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.fail('this failed')

        assert c.failure_reasons == ['this failed']

    def test_failure_reasons_multiple(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.fail('this failed')
            c.fail('this failed too')

        assert c.failure_reasons == ['this failed', 'this failed too']


class TestCheckStatus:
    def test_status_pass(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.assert_true(True)

        assert c.status == CheckStatus.PASS

    def test_status_fail(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.fail('this failed')

        assert c.status == CheckStatus.FAIL

    def test_status_no_data(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.get_value('.not.a.path')

        assert c.status == CheckStatus.PENDING

    def test_status_no_assertions(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            pass

        assert c.status == CheckStatus.PASS

    def test_fail_before_pass(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.assert_true(True)
            c.fail('this failed')

        assert c.status == CheckStatus.FAIL

    def test_no_data_before_pass(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.assert_true(True)
            c.get_value('.not.a.path')

        assert c.status == CheckStatus.PENDING

    def test_no_data_before_fail(self):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.fail('this failed')
            c.get_value('.not.a.path')

        assert c.status == CheckStatus.PENDING



from src.lunar_policy.nodepath import Segment, SegmentKind


class TestSegmentStrings:
    def test_dot_segment(self):
        segment = Segment(kind=SegmentKind.DOT, key='a')
        assert str(segment) == '.a'

    def test_dot_segment_with_index_and_key(self):
        segment = Segment(kind=SegmentKind.DOT, key='a', index=0)
        assert str(segment) == '.a[0]'

    def test_dot_segment_with_empty_key(self):
        segment = Segment(kind=SegmentKind.DOT, key='')
        assert str(segment) == '.'

    def test_lit_segment(self):
        segment = Segment(kind=SegmentKind.LIT, key='a')
        assert str(segment) == "['a']"

    def test_lit_segment_with_empty_key(self):
        segment = Segment(kind=SegmentKind.LIT, key='')
        assert str(segment) == "['']"

    def test_lit_segment_with_index_and_key(self):
        segment = Segment(kind=SegmentKind.LIT, key='a', index=0)
        assert str(segment) == "['a'][0]"



import json
import pytest
import tempfile

from src.lunar_policy import Check


class TestCheckBundle:
    @pytest.fixture(autouse=True)
    def setup_lunar_bundle(self, monkeypatch):
        test_data = {
            'bundle_info': {},
            'merged_blob': {
                'merged_true': True,
                'merged_false': False,
                'value_in_both_roots': 'hello world',
                'tricky_value': '.merged_true',
            },
            'metadata_instances': [
                {
                    'payload': {
                        'value_in_both_roots': 'hello moon',
                        'value_in_one_delta': 'one',
                        'value_in_two_deltas': 'two_a',
                    }
                },
                {
                    'payload': {
                        'value_in_both_roots': 'hello moon',
                        'value_in_two_deltas': 'two_b',
                    }
                },
            ],
        }

        with tempfile.NamedTemporaryFile(mode='w', delete=False) as f:
            json.dump(test_data, f)
            temp_path = f.name

        monkeypatch.setenv('LUNAR_BUNDLE_PATH', temp_path)

        yield

    def test_value_in_both_roots(self, capsys):
        with Check('test') as c:
            c.assert_equals(c.get_value('.value_in_both_roots'), 'hello world')
            for value in c.get_all_values('.value_in_both_roots'):
                c.assert_equals(value, 'hello moon')

        captured = capsys.readouterr()
        results = json.loads(captured.out)
        assert all(result['result'] == 'pass' for result in results['assertions'])
        assert results['paths'] == ['.value_in_both_roots']

    def test_value_in_single_delta(self, capsys):
        with Check('test') as c:
            for value in c.get_all_values('.value_in_one_delta'):
                c.assert_equals(value, 'one')

        captured = capsys.readouterr()
        results = json.loads(captured.out)
        assert all(result['result'] == 'pass' for result in results['assertions'])
        assert results['paths'] == ['.value_in_one_delta']

    def test_value_in_two_deltas(self, capsys):
        with Check('test') as c:
            for value in c.get_all_values('.value_in_two_deltas'):
                c.assert_contains(value, 'two')

        captured = capsys.readouterr()
        results = json.loads(captured.out)
        assert all(result['result'] == 'pass' for result in results['assertions'])
        assert results['paths'] == ['.value_in_two_deltas']



import json

from src.lunar_policy import Check, Node


class TestCheckExists:
    def test_exists_valid_path_workflows_finished(self):
        node = Node.from_component_json({'hi': 'there'}, bundle_info={'workflows_finished': True})

        with Check('test', node=node) as c:
            assert c.exists('.hi')

    def test_exists_valid_path_workflows_not_finished(self):
        node = Node.from_component_json({'hi': 'there'}, bundle_info={'workflows_finished': False})

        with Check('test', node=node) as c:
            assert c.exists('.hi')

    def test_exists_missing_path_workflows_finished(self):
        node = Node.from_component_json({}, bundle_info={'workflows_finished': True})

        with Check('test', node=node) as c:
            assert not c.exists('.missing')

    def test_exists_missing_path_workflows_not_finished(self, capsys):
        node = Node.from_component_json({}, bundle_info={'workflows_finished': False})

        with Check('test', node=node) as c:
            c.exists('.missing')
            c.assert_equals('should not run', 'should not run')

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert len(results) == 1
        assert results[0]['result'] == 'no-data'
        assert '.missing' in results[0]['failure_message']
        assert results[0]['op'] == 'fail'



import json
import semver

from src.lunar_policy import Check, Node


class TestCheckAssertions:
    def test_simple_value(self, capsys):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            c.assert_true(True)

        captured = capsys.readouterr()
        result = json.loads(captured.out)['assertions'][0]

        assert result['op'] == 'true'
        assert result['args'] == ['True']
        assert result['result'] == 'pass'
        assert 'failure_message' not in result

    def test_simple_path(self, capsys):
        node = Node.from_component_json(
            {
                'value': True,
            }
        )

        with Check('test', node=node) as c:
            c.assert_true(c.get_value('.value'))

        captured = capsys.readouterr()
        result = json.loads(captured.out)['assertions'][0]

        assert result['op'] == 'true'
        assert result['args'] == ['True']
        assert result['result'] == 'pass'
        assert 'failure_message' not in result

    def test_multiple_mixed_assertions_in_check(self, capsys):
        node = Node.from_component_json(
            {
                'value': True,
            }
        )

        with Check('test', node=node) as c:
            c.assert_true(True)
            c.assert_true(c.get_value('.value'))

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_true_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': True,
            }
        )

        with Check('test', node=node) as c:
            c.assert_true(True)
            c.assert_true(c.get_value('.value'))

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_false_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': False,
            }
        )

        with Check('test', node=node) as c:
            c.assert_false(False)
            c.assert_false(c.get_value('.value'))

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_equals_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': 1,
            }
        )

        with Check('test', node=node) as c:
            c.assert_equals(1, 1)
            c.assert_equals(c.get_value('.value'), 1)

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_greater_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': 1,
            }
        )

        with Check('test', node=node) as c:
            c.assert_greater(2, 1)
            c.assert_greater(c.get_value('.value'), 0)

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_greater_or_equal_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': 1,
            }
        )

        with Check('test', node=node) as c:
            c.assert_greater_or_equal(2, 1)
            c.assert_greater_or_equal(1, 1)
            c.assert_greater_or_equal(c.get_value('.value'), 0)
            c.assert_greater_or_equal(c.get_value('.value'), 1)

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_less_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': 1,
            }
        )

        with Check('test', node=node) as c:
            c.assert_less(1, 2)
            c.assert_less(c.get_value('.value'), 2)

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_less_or_equal_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': 1,
            }
        )

        with Check('test', node=node) as c:
            c.assert_less_or_equal(1, 2)
            c.assert_less_or_equal(1, 1)
            c.assert_less_or_equal(c.get_value('.value'), 2)
            c.assert_less_or_equal(c.get_value('.value'), 1)

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_contains_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': 'hello',
            }
        )

        with Check('test', node=node) as c:
            c.assert_contains('hello', 'e')
            c.assert_contains('hello', 'hello')
            c.assert_contains('hello', '')
            c.assert_contains(c.get_value('.value'), 'e')

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_match_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': 'hello',
            }
        )

        with Check('test', node=node) as c:
            c.assert_match('hello', '.*ell.*')
            c.assert_match('hello', '.*hello.*')
            c.assert_match('hello', '.*')
            c.assert_match(c.get_value('.value'), '.*ell.*')

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_foreign_type_comparison(self, capsys):
        node = Node.from_component_json({})

        with Check('test', node=node) as c:
            v1 = semver.Version.parse('1.0.0')
            v2 = semver.Version.parse('2.0.0')

            c.assert_greater_or_equal(v2, v1)
            c.assert_less_or_equal(v1, v2)

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)

    def test_exists_assertion(self, capsys):
        node = Node.from_component_json(
            {
                'value': 1,
            }
        )

        with Check('test', node=node) as c:
            c.assert_exists('.value')

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert all(result['result'] == 'pass' for result in results)



import json

from src.lunar_policy import Check, Node


class TestCheckGet:
    def test_one_check_output_with_prefix(self, capsys, monkeypatch):
        node = Node.from_component_json({'hi': 'there'})
        prefix = 'LUNAR_SNIPPETS: '

        monkeypatch.setenv('LUNAR_LOG_PREFIX', prefix)

        with Check('test', node=node) as c:
            c.get_value('.hi')

        captured = capsys.readouterr()
        assert captured.out.startswith(prefix), f'Missing prefix: {captured}'

        json_part = captured.out[len(prefix) :]
        try:
            json.loads(json_part)
        except Exception as e:
            raise AssertionError(f'Invalid JSON: {json_part!r}') from e

    def test_multiple_checks_output_with_prefix(self, capsys, monkeypatch):
        node = Node.from_component_json({'hi': 'there'})
        prefix = 'LUNAR_SNIPPETS: '

        monkeypatch.setenv('LUNAR_LOG_PREFIX', prefix)

        with Check('test', node=node) as c:
            c.get_value('.hi')

        with Check('test', node=node) as c:
            c.get_value('.hi')

        captured = capsys.readouterr()
        lines = captured.out.splitlines()
        assert len(lines) == 2, f'Expected 2 lines, got {len(lines)}'

        for line in captured.out.splitlines():
            assert line.startswith(prefix), f'Missing prefix: {line!r}'
            json_part = line[len(prefix) :]
            try:
                json.loads(json_part)
            except Exception as e:
                raise AssertionError(f'Invalid JSON: {json_part!r}') from e



import json

from src.lunar_policy import Check, Node


class TestCheckSkipped:
    def test_skip_no_reason(self, capsys):
        node = Node.from_component_json({})
        with Check('test', node=node) as c:
            c.skip()

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert len(results) == 1
        assert results[0]['result'] == 'skipped'
        assert results[0]['op'] == 'skip'

    def test_skip_with_reason(self, capsys):
        node = Node.from_component_json({})
        with Check('test', node=node) as c:
            c.skip('idk, just skip it bro')

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert len(results) == 1
        assert results[0]['result'] == 'skipped'
        assert results[0]['op'] == 'skip'
        assert results[0]['failure_message'] == 'idk, just skip it bro'

    def test_skip_after_assertions(self, capsys):
        node = Node.from_component_json({})
        with Check('test', node=node) as c:
            c.assert_true(True)
            c.skip('idk, just skip it bro')

        captured = capsys.readouterr()
        results = json.loads(captured.out)['assertions']
        assert len(results) == 1
        assert results[0]['result'] == 'skipped'
        assert results[0]['op'] == 'skip'
        assert results[0]['failure_message'] == 'idk, just skip it bro'

    def test_skip_exits_early(self, capsys):
        node = Node.from_component_json({})
        with Check('test', node=node) as c:
            c.skip('idk, just skip it bro')
            raise Exception('should not run')
